(********************************************************************
 * COPYRIGHT -- Jenny Science AG
 ********************************************************************
 * Library: JsMcLib
 * File: JsMcLib.fun
 * Author: Olivia Wspi
 * Created: August 23, 2011
 ********************************************************************
 * Functions and function blocks of library JsMcLib
 ********************************************************************
 * History:
 *
 * Version  | Date             | Author        | Description
 * ---------------------------------------------------------------------------------
 *    1.00  | 2011-sep-14   | O.Wspi    | -construction
 *    2.00  | 2014-jun-14    | A.West    | -JS_MC_Cyclic splitted off into
 *             |                        |                  |   JS_MC_CyclicIn and JS_MC_CyclicOut
 *             |                        |                  | -JS_MC_AxisErrorCollector added
 *             |                        |                  | -Interface of JS_MC_ReadAxisError adopted
 *             |                        |                  | -Interface of JS_MC_Reference adopted
 *             |                        |                  | -Interface of JS_MC_ReadStatus adopted
 *             |                        |                  | -Interface of JS_MC_ReadParameter adopted
 *             |                        |                  | -Interface of JS_MC_WriteParameter adopted
 *             |                        |                  | -Unused internal variables deleted
 *             |                        |                  | -Functionblock JS_MC_ReadWriteParameter and JS_MC_MotorParameter deleted
 *             |                        |                  | -Interface of JS_MC_Halt and JS_MC_Stop adopted (Scurve -input deleted)
 *             |                        |                  | -Description for all function blocks updated
 *    2.01  | 2014-jul-03    | A.West    | -Init of several variables implemented
 *             |                        |                  | -Datatype of JS_MC_INIT.OperationMode changed to USINT
 *             |                        |                  | -Datatype of JS_MC_Stop.Deceleration changed to UDINT
 *             |                        |                  | -Datatype of JS_MC_Halt.Deceleration changed to UDINT
 *             |                        |                  | -Datatype of JS_MC_MoveAbsolute.Position changed to DINT
 *             |                        |                  | -Datatype of JS_MC_MoveAbsolute.Velocity changed to UDINT
 *             |                        |                  | -Datatype of JS_MC_MoveAbsolute.Acceleration changed to UDINT
 *             |                        |                  | -Datatype of JS_MC_MoveAbsolute.Scurve changed to UDINT
 *             |                        |                  | -Datatype of JS_MC_MoveRelative.Distance changed to DINT
 *             |                        |                  | -Datatype of JS_MC_MoveRelative.Velocity changed to UDINT
 *             |                        |                  | -Datatype of JS_MC_MoveRelative.Acceleration changed to UDINT
 *             |                        |                  | -Datatype of JS_MC_MoveRelative.Scurve changed to UDINT
 *             |                        |                  | -Datatype of JS_MC_MoveCyclicPosition.Position changed to DINT
 *    2.10  | 2014-jul-15    | A.West    | -Several variables datatypes adopted
 *             |                        |                  | -Function block JS_MC_JogVelocity added
 *             |                        |                  | -JS_MC_ReadStatus adopted (now with output ContinousMotion)
*********************************************************************)
(*In and Outputs for the AxisErrorCollector*)

FUNCTION_BLOCK JS_MC_AxisErrorCollector
	VAR_INPUT
		Axis : UDINT; (*The axis reference handle*)
		Enable : BOOL; (*As long as "Enable" is TRUE, the function block determines error information*)
	END_VAR
	VAR_OUTPUT
		Valid : BOOL; (*A valid set of outputs is available at the function block*)
		Busy : BOOL; (*The function block is not finished*)
		Error : BOOL; (*Error occurred within this function block (JS_MC_AxisErrorCollector)*)
		ErrorID : UINT; (*Error number of this function block (JS_MC_AxisErrorCollector)*)
		FunctionBlockError : BOOL; (*A Jenny Science function block is in error state*)
		AxisError : BOOL; (*The axis is in error state*)
		AxisWarning : BOOL; (*The axis is in warning state*)
		Errorstop : BOOL; (*The current PCLopen State is "Errorstop"*)
	END_VAR
	VAR
		IS : JS_MC_AxisErrorCollector_IS; (*Internal variables*)
	END_VAR
END_FUNCTION_BLOCK
(*In and Outputs for the CyclicIn (Function block needs to be called at the top of a cyclic program, before the call of all other function blocks)*)

FUNCTION_BLOCK JS_MC_CyclicIn
	VAR_INPUT
		Axis : UDINT; (*The axis reference handle*)
		Enable : BOOL; (*As long as "Enable" is TRUE, cyclic data are received from the power link bus ("Enable" must always be TRUE during operation)*)
	END_VAR
	VAR_OUTPUT
		Valid : BOOL; (*Cyclic power link communication is valid (it is not allowed to enable or execute other function blocks unless valid is TRUE)*)
		Error : BOOL; (*Error occurred within function block*)
		ErrorID : UINT; (*Error number*)
	END_VAR
	VAR
		IS : JS_MC_CyclicIn_IS; (*Internal variables*)
	END_VAR
END_FUNCTION_BLOCK
(*In and Outputs for the CyclicOut (function block needs to be called at the end of a cyclic program, after the call of all other function blocks)*)

FUNCTION_BLOCK JS_MC_CyclicOut
	VAR_INPUT
		Axis : UDINT; (*The axis reference handle*)
	END_VAR
END_FUNCTION_BLOCK
(*In and Outputs for the ForceCalibration*)

FUNCTION_BLOCK JS_MC_ForceCalibration
	VAR_INPUT
		Axis : UDINT; (*The axis reference handle*)
		Execute : BOOL; (*Start the ForceCalibration at rising edge*)
		StartPosition : DINT; (*Start position for the force calibration [increment]*)
		EndPosition : DINT; (*End position for the force calibration [increment]*)
		IterativeFcDisable : BOOL; (*IF 0, takes values from previous FC call to improve calibration. IF 1, ignores calibration values from last FC call,*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Commanded position reached*)
		Busy : BOOL; (*The function block is not finished*)
		CommandAborted : BOOL; (*Function block is aborted by another command*)
		Error : BOOL; (*Error occurred within function block*)
		ErrorID : UINT; (*Error number*)
	END_VAR
	VAR
		IS : JS_MC_ForceCalibration_IS; (*Internal variables*)
	END_VAR
END_FUNCTION_BLOCK
(*In and Outputs for the Halt*)

FUNCTION_BLOCK JS_MC_Halt
	VAR_INPUT
		Axis : UDINT; (*The axis reference handle*)
		Execute : BOOL; (*Start the action at rising edge*)
		Deceleration : UDINT; (*Value of the deceleration [increment/s2] *)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Zero velocity is reached*)
		Busy : BOOL; (*The function block is not finished*)
		CommandAborted : BOOL; (*Function block is aborted by another command*)
		Error : BOOL; (*Error occurred within function block*)
		ErrorID : UINT; (*Error number*)
	END_VAR
	VAR
		IS : JS_MC_Halt_IS; (*Internal variables*)
	END_VAR
END_FUNCTION_BLOCK
(*Only for internal usage. In and Outputs for the HandleModeOfOperation*)

FUNCTION_BLOCK JS_MC_HndlModeOfOperation
	VAR_INPUT
		Axis : UDINT; (*The axis reference handle*)
		ToStandstillMode : BOOL; (*Request to switch to the mode needed for Stanstill*)
		ForceCalibActive : BOOL; (*If set in continuous motion, mode of operation goes to 0 for Forceteq instead of -1 for JogVelocity*)
	END_VAR
	VAR_OUTPUT
		Error : BOOL; (*Communication Error occurred within function block*)
		StandstillModeSet : BOOL; (*In Mode of Operation for Standstill*)
		ReferenceModeSet : BOOL; (*Mode 6 for Reference set*)
		ContMotModeSet : BOOL; (*Mode -1 for JogVelocity Set*)
		DisabledModeSet : BOOL; (*Mode 0 (no Mode) set*)
	END_VAR
	VAR
		IS : JS_MC_HndlModeOfOperation_IS; (*Internal variables*)
	END_VAR
END_FUNCTION_BLOCK
(*In and Outputs for the Init*)

FUNCTION_BLOCK JS_MC_Init (*Infra structure:Init*)
	VAR_INPUT
		AdrIoMap : REFERENCE TO JS_MC_IoMap; (*Address of IO-Map struct*)
		pDevice : UDINT; (*Powerlink interface address given as a pointer, e.g., "SLx.SSy.IFz"*)
		Node : USINT; (*Node ID (must be equal to card identifier in the Xenax servocontroller)*)
		OperationMode : SINT; (*General mode of operation:<br>
jsmcMODE_PROFILE_POSITION<br>
jsmcMODE_CYCLIC_SYNC*)
		EnforceReferenceDrive : BOOL; (*false: axis can be refernced onyl once. true: axis will refernce even when the axis is already referenced*)
	END_VAR
	VAR_OUTPUT
		Axis : UDINT; (*The axis reference handle*)
	END_VAR
	VAR
		IS : JS_MC_Init_IS; (*Internal variables*)
	END_VAR
END_FUNCTION_BLOCK
(*In and Outputs for the JogVelocity*)

FUNCTION_BLOCK JS_MC_JogVelocity
	VAR_INPUT
		Axis : UDINT; (*The axis reference handle*)
		Enable : BOOL; (*As long as "Enable" is TRUE, the axis is in Continuous Motion mode (jog drive possible)*)
		Velocity : UDINT; (*Value of maximum velocity [increment/s]<br>

Note:<br>
This value can also be changed while a movement is taking place*)
		Acceleration : UDINT; (*Value of maximum acceleration [increment/s2]<br>

Note:<br>
This value can also be changed while a movement is taking place (new value is used at the next velocity change)*)
		Deceleration : UDINT; (*Value of maximum deceleration [increment/s2]<br>

Note:<br>
This value can also be changed while a movement is taking place (new value is used at the next velocity change)*)
		JogPositive : BOOL; (*Executes a movement in the positive direction*)
		JogNegative : BOOL; (*Executes a movement in the negative direction*)
	END_VAR
	VAR_OUTPUT
		Active : BOOL; (*The function block is active, possible to execute movements*)
		Busy : BOOL; (*The function block is not finished*)
		CommandAborted : BOOL; (*Function block is aborted by another command*)
		Error : BOOL; (*Error occurred within function block*)
		ErrorID : UINT; (*Error number*)
		Jogging : BOOL; (*Movement being carried out*)
	END_VAR
	VAR
		IS : JS_MC_JogVelocity_IS; (*Internal variables*)
	END_VAR
END_FUNCTION_BLOCK
(*In and Outputs for the MoveAbsolute*)

FUNCTION_BLOCK JS_MC_MoveAbsolute
	VAR_INPUT
		Axis : UDINT; (*The axis reference handle*)
		Execute : BOOL; (*Start move at rising edge*)
		Position : DINT; (*Target position for the motion [increment]*)
		Velocity : UDINT; (*Value of maximum velocity [increment/s] (not necessarily reached)*)
		Acceleration : UDINT; (*Value of maximum acceleration [increment/s2] (not necessarily reached)*)
		Scurve : UDINT; (*Value of S-curve parameter during the acceleration [%]*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Commanded position reached*)
		Busy : BOOL; (*The function block is not finished*)
		CommandAborted : BOOL; (*Function block is aborted by another command*)
		Error : BOOL; (*Error occurred within function block*)
		ErrorID : UINT; (*Error number*)
	END_VAR
	VAR
		IS : JS_MC_MoveAbsolute_IS; (*Internal variables*)
	END_VAR
END_FUNCTION_BLOCK
(*In and Outputs for the MoveCyclicPosition*)

FUNCTION_BLOCK JS_MC_MoveCyclicPosition
	VAR_INPUT
		Axis : UDINT; (*The axis reference handle*)
		Enable : BOOL; (*As long as "Enable" is TRUE, the cyclic position is transferred to the axis*)
		Position : DINT; (*Cyclic position to be transferred to the axis  [increment]*)
	END_VAR
	VAR_OUTPUT
		Valid : BOOL; (*A valid set of outputs is available at the function block*)
		CommandAborted : BOOL; (*Command is aborted by switching off power (only possibility to abort)*)
		Error : BOOL; (*Error occurred within function block*)
		ErrorID : UINT; (*Error number*)
	END_VAR
	VAR
		IS : JS_MC_MoveCyclicPosition_IS; (*Internal variables*)
	END_VAR
END_FUNCTION_BLOCK
(*In and Outputs for the MoveRelative*)

FUNCTION_BLOCK JS_MC_MoveRelative
	VAR_INPUT
		Axis : UDINT; (*The axis reference handle*)
		Execute : BOOL; (*Start move at rising edge*)
		Distance : DINT; (*Target distance for the motion [increment]*)
		Velocity : UDINT; (*Value of maximum velocity [increment/s] (not necessarily reached)*)
		Acceleration : UDINT; (*Value of maximum acceleration [increment/s2] (not necessarily reached)*)
		Scurve : UDINT; (*Value of S-curve parameter during the acceleration [%]*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Commanded distance reached*)
		Busy : BOOL; (*The function block is not finished*)
		CommandAborted : BOOL; (*Function block is aborted by another command*)
		Error : BOOL; (*Error occurred within function block*)
		ErrorID : UINT; (*Error number*)
	END_VAR
	VAR
		IS : JS_MC_MoveRelative_IS; (*Internal variables*)
	END_VAR
END_FUNCTION_BLOCK
(*In and Outputs for the Power*)

FUNCTION_BLOCK JS_MC_Power
	VAR_INPUT
		Axis : UDINT; (*The axis reference handle*)
		Enable : BOOL; (*As long as "Enable" is TRUE, the drive power stage is enabled*)
	END_VAR
	VAR_OUTPUT
		Status : BOOL; (*Effective status of the power stage*)
		Valid : BOOL; (*A valid set of outputs is available at the function block*)
		Error : BOOL; (*Error occurred within function block*)
		ErrorID : UINT; (*Error number*)
	END_VAR
	VAR
		IS : JS_MC_Power_IS; (*Internal variables*)
	END_VAR
END_FUNCTION_BLOCK
(*In and Outputs for the ReadActualCurrent*)

FUNCTION_BLOCK JS_MC_ReadActualCurrent
	VAR_INPUT
		Axis : UDINT; (*The axis reference handle*)
		Enable : BOOL; (*As long as "Enable" is TRUE, the actual current is read continuously*)
	END_VAR
	VAR_OUTPUT
		Valid : BOOL; (*A valid set of outputs is available at the function block*)
		Error : BOOL; (*Error occurred within function block*)
		ErrorID : UINT; (*Error number*)
		Current : INT; (*Actual motor current [mA]*)
	END_VAR
	VAR
		IS : JS_MC_ReadActualCurrent_IS; (*Internal variables*)
	END_VAR
END_FUNCTION_BLOCK
(*In and Outputs for the ReadActualPosition*)

FUNCTION_BLOCK JS_MC_ReadActualPosition
	VAR_INPUT
		Axis : UDINT; (*The axis reference handle*)
		Enable : BOOL; (*As long as "Enable" is TRUE, the actual position is read continuously*)
	END_VAR
	VAR_OUTPUT
		Valid : BOOL; (*A valid set of outputs is available at the function block*)
		Error : BOOL; (*Error occurred within function block*)
		ErrorID : UINT; (*Error number*)
		Position : DINT; (*Actual position of the axis [increment]*)
	END_VAR
END_FUNCTION_BLOCK
(*In and Outputs for the ReadAxisError*)

FUNCTION_BLOCK JS_MC_ReadAxisError
	VAR_INPUT
		Axis : UDINT; (*The axis reference handle*)
		Enable : BOOL; (*As long as "Enable" is TRUE, the function block can be used to read out axis and function block errors*)
		Acknowledge : BOOL; (*Acknowledges the error record currently displayed<br>
<br>
Note:<br>
Only the displayed error record is acknowledged. The errors at the axis or the function blocks itself need to be acknowledged directly at the axis or the corresponding function block*)
		DataAddress : UDINT; (*Address for the error text string (user variable)*)
		DataLength : UINT; (*Length of the error text string (at least 50 characters)*)
		DataObjectName : STRING[12]; (*Name of error text module*)
	END_VAR
	VAR_OUTPUT
		Valid : BOOL; (*A valid set of outputs is available at the function block<br>
<br>
Note:<br>
This output is set to FALSE while an error is being acknowledged or error text is being read.*)
		Busy : BOOL; (*The function block is not finished<br>
<br>
Note:<br>
This output is set to TRUE while error text or axis error number is read*)
		Error : BOOL; (*Error occurred within this function block (JS_MC_ReadAxisError)*)
		ErrorID : UINT; (*Error number of this function block (JS_MC_ReadAxisError)*)
		ErrorRecordAvailable : BOOL; (*Set if a new error record is displayed in the "ErrorRecord" output*)
		ErrorRecord : JS_MC_ErrorRecord; (*Error record including additional information (see data type JS_MC_ErrorRecord)*)
		FunctionBlockErrorCount : UINT; (*Number of pending function block errors to display*)
		AxisErrorCount : UINT; (*Number of pending axis errors to display*)
		AxisWarningCount : UINT; (*Number of pending axis warnings to display*)
	END_VAR
	VAR
		IS : JS_MC_ReadAxisError_IS; (*Internal variables*)
	END_VAR
END_FUNCTION_BLOCK
(*In and Outputs for the ReadDigitalInput*)

FUNCTION_BLOCK JS_MC_ReadDigitalInput
	VAR_INPUT
		Axis : UDINT; (*The axis reference handle*)
		Enable : BOOL; (*As long as "Enable" is TRUE, the digital inputs are read continuously*)
	END_VAR
	VAR_OUTPUT
		Valid : BOOL; (*A valid set of outputs is available at the function block*)
		Error : BOOL; (*Error occurred within function block*)
		ErrorID : UINT; (*Error number*)
		DigitalInput : UINT; (*The value of digital inputs (bit-coded)*)
	END_VAR
	VAR
		IS : JS_MC_ReadDigitalInput_IS; (*Internal variables*)
	END_VAR
END_FUNCTION_BLOCK
(*In and Outputs for the ReadDigitalOutput*)

FUNCTION_BLOCK JS_MC_ReadDigitalOutput
	VAR_INPUT
		Axis : UDINT; (*The axis reference handle*)
		Enable : BOOL; (*As long as "Enable" is TRUE, the digital outputs are read continuously*)
	END_VAR
	VAR_OUTPUT
		Valid : BOOL; (*A valid set of outputs is available at the function block*)
		Error : BOOL; (*Error occurred within function block*)
		ErrorID : UINT; (*Error number*)
		DigitalOutput : USINT; (*The value of digital outputs (bit-coded)*)
	END_VAR
	VAR
		IS : JS_MC_ReadDigitalOutput_IS; (*Internal variables*)
	END_VAR
END_FUNCTION_BLOCK
(*In and Outputs for the ReadParameter*)

FUNCTION_BLOCK JS_MC_ReadParameter
	VAR_INPUT
		Axis : UDINT; (*The axis reference handle*)
		Enable : BOOL; (*As long as "Enable" is TRUE, the desired data object is read out continuously*)
		DataObject : UINT; (*Desired data object to be read out (according CANopen communication profile)*)
		SubID : USINT; (*SubID of the desired data object to be read out*)
		DataLength : USINT; (*Data length of the desired data object to be read out in bytes*)
	END_VAR
	VAR_OUTPUT
		Valid : BOOL; (*A valid set of outputs is available at the function block*)
		Error : BOOL; (*Error occurred within function block*)
		ErrorID : UINT; (*Error number*)
		Value : DINT; (*Value of the desired data object after read out*)
	END_VAR
	VAR
		IS : JS_MC_ReadParameter_IS; (*Internal variables*)
	END_VAR
END_FUNCTION_BLOCK
(*In and Outputs for the ReadPSR*)

FUNCTION_BLOCK JS_MC_ReadPSR
	VAR_INPUT
		Axis : UDINT; (*The axis reference handle*)
		Enable : BOOL; (*As long as "Enable" is TRUE, the actual Process Status Register of the XENAX controller is read continuously*)
	END_VAR
	VAR_OUTPUT
		Valid : BOOL; (*A valid set of outputs is available at the function block*)
		Error : BOOL; (*Error occurred within function block*)
		ErrorID : UINT; (*Error number*)
		ProcessStatusRegister : JS_MC_Init_IS_ProcStat; (*Process Status Register of the XENAX controller (see data type JS_MC_Init_IS_ProcStat)*)
	END_VAR
END_FUNCTION_BLOCK
(*In and Outputs for the ReadStatus*)

FUNCTION_BLOCK JS_MC_ReadStatus
	VAR_INPUT
		Axis : UDINT; (*The axis reference handle*)
		Enable : BOOL; (*As long as "Enable" is TRUE, the axis status is read continuously*)
	END_VAR
	VAR_OUTPUT
		Valid : BOOL; (*A valid set of outputs is available at the function block*)
		Error : BOOL; (*Error occurred within function block*)
		ErrorID : UINT; (*Error number*)
		Errorstop : BOOL; (*An error has occurred. Use JS_MC_Reset to acknowledge errors*)
		Disabled : BOOL; (*JS_MC_Power has not powered the axis, or an error had been acknowledged by JS_MC_Reset and the axis has been turned off*)
		Stopping : BOOL; (*JS_MC_Stop is active*)
		Standstill : BOOL; (*Motion is not active on the drive*)
		DiscreteMotion : BOOL; (*Axis is in motion due to one of the following function blocks:<br>
JS_MC_MoveAbsolute<br>
JS_MC_MoveRelative<br>
JS_MC_Halt*)
		Reference : BOOL; (*JS_MC_Reference has started referencing the axis*)
		ContinuousMotion : BOOL; (*Axis is in motion due to the following function block:<br>
JS_MC_JogVelocity*)
	END_VAR
	VAR
		IS : JS_MC_ReadStatus_IS; (*Internal variables*)
	END_VAR
END_FUNCTION_BLOCK
(*In and Outputs for the Reference*)

FUNCTION_BLOCK JS_MC_Reference
	VAR_INPUT
		Axis : UDINT; (*The axis reference handle*)
		Execute : BOOL; (*Start reference at rising edge*)
		ReferenceMode : USINT; (*Reference modes for linear drives:<br>
jsmcREFERENCE_MODE_REF_MARK_POS<br>
jsmcREFERENCE_MODE_REF_MARK_NEG<br>
jsmcREFERENCE_GANTRY_POS<br>
jsmcREFERENCE_GANTRY_NEG<br>
jsmcREFERENCE_GANTRY_POS_REV_MOT<br>
jsmcREFERENCE_GANTRY_NEG_REV_MOT<br>
<br>
Reference modes for rotative drives:<br>
jsmcREFERENCE_MODE_CLOCK_C<br>
jsmcREFERENCE_MODE_CLOCK<br>
jsmcREFERENCE_MODE_COUNTER_CLOCK<br>
jsmcREFERENCE_MODE_CONTINUE_CLO*)
		ReferenceSpeedRot : UDINT; (*Reference speed [increment/s] (only for rotative drives)*)
		ZMarkSpeedRot : UDINT; (*Z-Mark speed [increment/s] (only for rotative drives)*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Reference procedure has finished successfully*)
		Busy : BOOL; (*The function block is not finished*)
		CommandAborted : BOOL; (*Function block is aborted by another command*)
		Error : BOOL; (*Error occurred within function block*)
		ErrorID : UINT; (*Error number*)
	END_VAR
	VAR
		IS : JS_MC_Reference_IS; (*Internal variables*)
	END_VAR
END_FUNCTION_BLOCK
(*In and Outputs for the Reset*)

FUNCTION_BLOCK JS_MC_Reset
	VAR_INPUT
		Axis : UDINT; (*The axis reference handle*)
		Execute : BOOL; (*Reset the axis at the rising edge*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Standstill or Disabled state is reached*)
		Busy : BOOL; (*The function block is not finished*)
		Error : BOOL; (*Error occurred within function block*)
		ErrorID : UINT; (*Error number*)
	END_VAR
	VAR
		IS : JS_MC_Reset_IS; (*Internal variables*)
	END_VAR
END_FUNCTION_BLOCK
(*In and Outputs for the WriteParameter*)

FUNCTION_BLOCK JS_MC_SetPDO
	VAR_INPUT
		Axis : UDINT; (*The axis reference handle*)
		Execute : BOOL; (*Sets the value at the rising edge*)
		Index : SINT; (*only index 5 is valid --> I_ForceLimit*)
		Value : DINT; (*Value to set*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*The function block is finished*)
		Busy : BOOL; (*The function block is not finished*)
		Error : BOOL; (*Error occurred within function block*)
		ErrorID : UINT; (*Error number*)
	END_VAR
	VAR
		IS : JS_MC_WriteParameter_IS; (*Internal variables*)
	END_VAR
END_FUNCTION_BLOCK
(*In and Outputs for the Stop*)

FUNCTION_BLOCK JS_MC_Stop
	VAR_INPUT
		Axis : UDINT; (*The axis reference handle*)
		Execute : BOOL; (*Start the action at rising edge*)
		Deceleration : UDINT; (*Value of the deceleration [increment/s2] *)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Zero velocity is reached*)
		Busy : BOOL; (*The function block is not finished*)
		CommandAborted : BOOL; (*Command is aborted by switching off power (only possibility to abort)*)
		Error : BOOL; (*Error occurred within function block*)
		ErrorID : UINT; (*Error number*)
	END_VAR
	VAR
		IS : JS_MC_Stop_IS; (*Internal variables*)
	END_VAR
END_FUNCTION_BLOCK
(*In and Outputs for the WriteDigitalOutput*)

FUNCTION_BLOCK JS_MC_WriteDigitalOutput
	VAR_INPUT
		Axis : UDINT; (*The axis reference handle*)
		Execute : BOOL; (*Writes the DigitalOutput value at the rising edge*)
		DigitalOutput : USINT; (*The value of digital outputs (bit-coded)*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Digital outputs are written*)
		Error : BOOL; (*Error occurred within function block*)
		ErrorID : UINT; (*Error number*)
	END_VAR
	VAR
		IS : JS_MC_WriteDigOut_IS; (*Internal variables*)
	END_VAR
END_FUNCTION_BLOCK
(*In and Outputs for the WriteParameter*)

FUNCTION_BLOCK JS_MC_WriteParameter
	VAR_INPUT
		Axis : UDINT; (*The axis reference handle*)
		Execute : BOOL; (*Writes the value at the rising edge*)
		DataObject : UINT; (*Desired data object to be written (according CANopen communication profile)*)
		SubID : USINT; (*SubID of the desired data object to be written*)
		DataLength : USINT; (*Data length of the desired data object to be written in bytes*)
		Value : DINT; (*Value to be written to the desired data object*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Value is written to the object*)
		Busy : BOOL; (*The function block is not finished*)
		Error : BOOL; (*Error occurred within function block*)
		ErrorID : UINT; (*Error number*)
	END_VAR
	VAR
		IS : JS_MC_WriteParameter_IS; (*Internal variables*)
	END_VAR
END_FUNCTION_BLOCK
