(***********************************************************************************
 * COPYRIGHT -- Jenny Science AG
 ***********************************************************************************
 * Library: JsMcLib
 * File: JsMcLib.typ
 * Author: Olivia Wspi
 * Created: August 23, 2011
 ***********************************************************************************
 * Data types of library JsMcLib
 ***********************************************************************************
 * History:
 *
 * Version  | Date             | Author        | Description
 * ---------------------------------------------------------------------------------
 *    1.00  | 2011-sep-14   | O.Wspi    | -construction
 *    2.00  | 2014-jun-10    | A.West    | -JS_MC_IoMapp with new variables extended
 *             |                        |                  | -JS_MC_Init_IS_ProcStat with new bits extended
 *             |                        |                  | -Define DS402State and PLCopenState instead
 *             |                        |                  |  of enRunState in JS_MC_Init.IS
 *             |                        |                  | -JS_MC_OperationMode defined
 *             |                        |                  | -Diverse IS-Blocks extended with bLastEnable
 *             |                        |                  | -Statemachine of JS_MC_CylcicIN extended
 *             |                        |                  | -Global ErrorInfo added (in JS_MC_INIT)
 *             |                        |                  | -Statemachine for JS_MC_ReadAxisError splitted off
 *             |                        |                  |  into two statemachines (Text read and SDO read)
 *             |                        |                  | -Diverse adaptions for changes in function blocks
 *    2.00  | 2014-jun-14    | A.West    | -Enumeratortype "JS_MC_OperationMode" deleted and replaced with constants
 *             |                        |                  | -Init of several variables
 *    2.10  | 2014-jul-15     | A.West    | -Several variable adopted for new functionallity
 *             |                        |                  | -Datatypes for JS_MC_JogVelocity added
 *    2.13  | 2016-jul-18     | A.West    | -String for szDevice enlarged
 *             |                        |                  | -Enumerators added for SPAD funcationality
 *    2.14  | 2018-mar-02  | H.Ammann  | -New state during PowerUp (enPOWER_W4_READY_FOR_DISABLE)
 ***********************************************************************************

 ***********************************************************************************)
(*Io Mapping *)

TYPE
	JS_MC_IoMap : 	STRUCT  (*Cyclic data*)
		bModuleOK : BOOL;
		udiS_CurveProfile_I2000In : UDINT;
		udiS_CurveProfile_I2000Out : UDINT;
		diI_ForceActual_I2005In : DINT;
		diProcessStatusRegister_I2006In : DINT;
		uiControlword_I6040In : UINT;
		uiControlword_I6040Out : UINT;
		uiStatusword_I6041In : UINT;
		diPositionActualValue_I6064In : DINT;
		udiFollowingErrorWindow_I6065In : UDINT;
		udiFollowingErrorWindow_I6065Out : UDINT;
		udiPositionWindow_I6067In : UDINT;
		udiPositionWindow_I6067Out : UDINT;
		uiI_ForceMax_I6073In : UINT;
		uiI_ForceMax_I6073Out : UINT;
		iMotorCurrentActualValue_I6078In : INT;
		diTargetPosition_I607AIn : DINT;
		diTargetPosition_I607AOut : DINT;
		udiProfileVelocity_I6081In : UDINT;
		udiProfileVelocity_I6081Out : UDINT;
		udiProfileAcceleration_I6083In : UDINT;
		udiProfileAcceleration_I6083Out : UDINT;
		diFollowErrorActualValue_I60F4In : DINT;
		udiDigitalInputs_I60FDIn : UDINT;
		udiPhysicalOutputs_I60FE_S01In : UDINT;
		udiPhysicalOutputs_I60FE_S01Out : UDINT;
	END_STRUCT;
END_TYPE

(*For the Cyclic*)

TYPE
	JS_MC_CyclicIn_IS : 	STRUCT  (*Internal structure for JS_MC_CyclicIn*)
		enRunState : JS_MC_CyclicIn_IE_Sma; (*Runstate of state machine*)
		bLastEnable : BOOL; (*Save of last enable*)
		stPDOMapData : ARRAY[0..13]OF JS_MC_CyclicIn_IS_PDOMapData; (*List of all PDO-Objects which might be neccessary to map*)
		usiPDOMappingCounter : USINT; (*Counter for PDO mapping check*)
		usiPDOMappingIndex : USINT; (*Index for PDO mapping check*)
		diDummyResult : DINT; (*Dummyresult for diverse checks (for example software version check)*)
		szDummyResult : STRING[31]; (*Dummyresult for diverse checks (for example software version check)*)
		instEplSDORead : EplSDORead; (*Instance for SDO readouts*)
		instAsIOPVInfo : AsIOPVInfo; (*Instance for PDO-Mapping check*)
	END_STRUCT;
	JS_MC_CyclicIn_IE_Sma : 
		( (*JS_MC_CyclicIn state machine*)
		enCYCIN_PDO_MAPPING_INIT,
		enCYCIN_CHECK_PDO_MAPPING_INIT,
		enCYCIN_CHECK_PDO_MAPPING_VERIFY,
		enCYCIN_READ_TYPE_MOTOR_INIT,
		enCYCIN_READ_TYPE_MOTOR_VERIFY,
		enCYCIN_READ_XENAX_FW_VER_INIT,
		enCYCIN_READ_XENAX_FW_VER_VERIFY,
		enCYCIN_READ_BUSMD_FW_VER_INIT,
		enCYCIN_READ_BUSMD_FW_VER_VERIFY,
		enCYCIN_READ_SPAD_SETTING_INIT,
		enCYCIN_READ_SPAD_SETTING_VERIFY,
		enCYCIN_CHECK_CYCLE_TIME,
		enCYCIN_INIT_COMPLETED,
		enCYCIN_ERROR
		);
	JS_MC_CyclicIn_IS_PDOMapData : 	STRUCT  (*Buffer for PDO mapping check*)
		ChannelName : STRING[65]; (*Channel name for PDO-Mapping (according iomap.xml-file)*)
		ProcessVariableName : STRING[33]; (*Process varaible name in IoMap struckt (f.ex. bModuleOk)*)
		AccessType : BOOL; (*PDO access type (FALSE = read, TRUE = write)*)
		IsMapped : BOOL; (*Indicates that the PDO is mapped correctly.*)
	END_STRUCT;
END_TYPE

(*For the Init*)

TYPE
	JS_MC_Init_IS : 	STRUCT  (*internal structure for JS_MC_Init*)
		pstIoMap : REFERENCE TO JS_MC_IoMap; (*IO-Map*)
		stStatWord : JS_MC_Init_IS_StatWord; (*DS402 Status word*)
		stCtrlWord : JS_MC_Init_IS_CtrlWord; (*DS402 Control word*)
		stProcStat : JS_MC_Init_IS_ProcStat; (*Process state register*)
		stPara : JS_MC_Init_IS_Para; (*Parmeters for cyclic transmition*)
		siGeneralMode : SINT; (*General mode of operation (Profile Position or Cyclic Sync)*)
		enDS402State : JS_MC_Init_IS_DS402State; (*Current DS402 state (calculated out ot the DS402 Status word)*)
		enPLCopenState : JS_MC_Init_IS_PLCopenState; (*Current PLCopen state (generated by the function blocks)*)
		enOldPLCopenState : JS_MC_Init_IS_PLCopenState; (*Last output PLCopen state*)
		stErrorInfo : JS_MC_Init_IS_ErrorInfo; (*Currently pending errors*)
		szDevice : STRING[15]; (*Device name of power link interface*)
		szTypeMotor : STRING[11]; (*Motor type (as string)*)
		udiTypeMotor : UDINT; (*Motor ID*)
		usiTypeMotor : USINT; (*Motor type (as integer)*)
		usiNode : USINT; (*Node number*)
		udiResponseTime : UDINT; (*Responsetime over powerlink in task cycles*)
		udiTaskCycleTime : UDINT; (*Task cycle time*)
		udiBusCycleTime : UDINT; (*Powerlink bus cycle time*)
		getIdent_obj : PLACTION_GET_IDENT_typ; (*Object for bus cycle time determination*)
		instRTInfo : RTInfo; (*RT info object*)
		bMC_Power_Status_Output : BOOL; (*Global copy of the JS_MC_Power.Status bit*)
		bMC_Power_Enable_Input : BOOL; (*Global copy of the JS_MC_Power.Enable bit*)
		stPDOMappingTable : JS_MC_Init_IS_PDOMappingTable; (*Table with PDO mapping info*)
		bCyclicCommOK : BOOL; (*Shows valid cyclic communication (global copy of the JS_MC_CyclicIn.Valid bit)*)
		EnforceReferenceDrive : BOOL; (*Forces the Axis to execute a reference drive even when the axis is already referenced *)
		usiDefaultReferenceMode : USINT; (*reference method used if user does not set it*)
		stHndlModeOfOperation : JS_MC_HndlModeOfOperation; (*Is used to switch the Mode of Operation*)
	END_STRUCT;
	JS_MC_Init_IS_ErrorInfo_FBEntry : 	STRUCT  (*Error entry for Function blocks*)
		enFBType : USINT; (*Function block which causes the error*)
		ErrorID : UINT; (*Error ID*)
	END_STRUCT;
	JS_MC_Init_IS_ErrorInfo : 	STRUCT  (*Infostruct with all error information from axis and function blocks*)
		stErrorInfo_FBEntry : ARRAY[0..4]OF JS_MC_Init_IS_ErrorInfo_FBEntry; (*Array for 5 function block error entries*)
		bError_FB : ARRAY[0..19]OF BOOL; (*Array for displaying function block errors*)
		usiErrorID_Axis : USINT; (*ErrorID of an axis error*)
		usiWarningID_Axis : USINT; (*ErrroID of an axis warning*)
	END_STRUCT;
	JS_MC_Init_IS_StatWord : 	STRUCT  (*StatusWord 6041 JS_MC_Init_IS_StatWord*)
		bRdyToSwitchOn : BOOL; (*Bit 0*)
		bSwitchedOn : BOOL; (*Bit 1*)
		bOperationEnabled : BOOL; (*Bit 2*)
		bFault : BOOL; (*Bit 3*)
		bVoltageEnabled : BOOL; (*Bit 4*)
		bQuickStop : BOOL; (*Bit 5*)
		bSwitchOnDisabled : BOOL; (*Bit 6*)
		bWarning : BOOL; (*Bit 7*)
		bManufacturSpecific1 : BOOL; (*Bit 8*)
		bRemote : BOOL; (*Bit 9*)
		bTargetReached : BOOL; (*Bit 10*)
		bSoftlLimitPosReached : BOOL; (*Bit 11*)
		bMoveAchived : BOOL; (*Bit 12*)
		bMoveError : BOOL; (*Bit 13*)
		bManufacturSpecific2 : BOOL; (*Bit 14*)
		bManufacturSpecific3 : BOOL; (*Bit 15*)
	END_STRUCT;
	JS_MC_Init_IS_CtrlWord : 	STRUCT  (*ControlWord 6040 JS_MC_Init_IS_CtrlWord*)
		bSwitchOn : BOOL; (*Bit 0*)
		bEnableVoltage : BOOL; (*Bit 1*)
		bQuickStop : BOOL; (*Bit 2*)
		bEnableOperation : BOOL; (*Bit 3*)
		bMoveStart : BOOL; (*Bit 4*)
		bNotUsed : BOOL; (*Bit 5*)
		bProfilePositionMode : BOOL; (*Bit 6*)
		bFaultReset : BOOL; (*Bit 7*)
		bHalt : BOOL; (*Bit 8*)
		bOpModeSpecific1 : BOOL; (*Bit 9*)
		bReserved : BOOL; (*Bit 10*)
		bManufacturSpecific1 : BOOL; (*Bit 11*)
		bManufacturSpecific2 : BOOL; (*Bit 12*)
		bManufacturSpecific3 : BOOL; (*Bit 13*)
		bManufacturSpecific4 : BOOL; (*Bit 14*)
		bManufacturSpecific5 : BOOL; (*Bit 15*)
	END_STRUCT;
	JS_MC_Init_IS_ProcStat : 	STRUCT  (*JS_MC_Init_IS_ProcStat*)
		bError : BOOL; (*Bit0*)
		bReference : BOOL; (*Bit1*)
		bInMotion : BOOL; (*Bit2*)
		bInPosition : BOOL; (*Bit3*)
		bEndOfProgram : BOOL; (*Bit4*)
		bInForce : BOOL; (*Bit5*)
		bInSector : BOOL; (*Bit6*)
		bForceInSector : BOOL; (*Bit7*)
		bInverterVoltagePresent : BOOL; (*Bit8*)
		bEndOfGantryInit : BOOL; (*Bit9*)
		bHwLimLeft : BOOL; (*Bit10*)
		bHWLimRight : BOOL; (*Bit11*)
		bEmergencyExit1 : BOOL; (*Bit12*)
		bEmergencyExit : BOOL; (*Bit13*)
		bCoggingRefDrivAct : BOOL; (*Bit14*)
		bIForceMaxLimitReached : BOOL; (*Bit 15*)
		bSTOPrimedHit : BOOL; (*Bit 16*)
		bSS1PrimedHit : BOOL; (*Bit 17*)
		bSS2Primed : BOOL; (*Bit 18*)
		bSS2Hit : BOOL; (*Bit 19*)
		bSLSPrimed : BOOL; (*Bit 20*)
		bSLSSpeedHit : BOOL; (*Bit 21*)
		bSLSPositionHit : BOOL; (*Bit 22*)
		bWarning : BOOL; (*Bit 23*)
		bInformation : BOOL; (*Bit 24*)
	END_STRUCT;
	JS_MC_Init_IS_Para : 	STRUCT  (*JS_MC_Init_IS_Para*)
		diActPos : DINT; (*Actual Position*)
		iActCur : INT; (*Actual Motor Current*)
		udiActIn : UDINT; (*Actual Digital Inputs*)
		udiActOut : UDINT; (*Actual Digital Outputs*)
		diSetPos : DINT; (*Set Position*)
		udiSetVel : UDINT; (*Set Velocity*)
		udiSetAcc : UDINT; (*Set Acceleration*)
		udiSetScurve : UDINT; (*Set Scurve*)
		udiSetOut : UDINT; (*Set Digital Outputs*)
		uiLimit_I_Force : UINT; (*Set Limit_I_Force*)
	END_STRUCT;
	JS_MC_Init_IS_DS402State : 
		( (*States according DS402 state machine*)
		enNOT_READY_TO_SWITCH_ON, (*NOT_READY_TO_SWITCH_ON*)
		enSWITCH_ON_DISABLED, (*SWITCH_ON_DISABLED*)
		enREADY_TO_SWITCH_ON, (*READY_TO_SWITCH_ON*)
		enSWITCHED_ON, (*SWITCHED_ON*)
		enOPERATION_ENABLE, (*OPERATION_ENABLE*)
		enQUICK_STOP_ACTIV, (*QUICK_STOP_ACTIV*)
		enFAULT_REACTION_ACTIVE, (*FAULT_REACTION_ACTIVE*)
		enFAULT (*FAULT*)
		);
	JS_MC_Init_IS_PLCopenState : 
		( (*States according PLCopen state machine*)
		enDISABLED, (*DISABLED*)
		enSTANDSTILL, (*STANDSTILL*)
		enREFERENCE, (*REFERENCE*)
		enERRORSTOP, (*ERRORSTOP*)
		enSTOPPING, (*STOPPING*)
		enDISCRETE_MOTION, (*DISCRETE_MOTION*)
		enCONTINUOUS_MOTION (*CONTINUOUS_MOTION*)
		);
	JS_MC_Init_IS_PDOMappingTable : 	STRUCT  (*Table with PDO mapping info*)
		bMandatory : BOOL; (*All mandatory PDOs mapped*)
		bForProfilePositionMode : BOOL; (*PDOs mapped for profile position mode*)
		bForReadCurrent : BOOL; (*PDOs mapped for read current*)
		bForReadInputs : BOOL; (*PDOs mapped for read inputs*)
		bForReadOutputs : BOOL; (*PDOs mapped for read outputs*)
		bForWriteOutputs : BOOL; (*PDOs mapped for write outputs*)
		bForWriteLimitIForce : BOOL; (*PDOs Mapped for limit I froce*)
	END_STRUCT;
END_TYPE

(*For the Power*)

TYPE
	JS_MC_Power_IS : 	STRUCT  (*internal structure for JS_MC_Power*)
		enRunState : JS_MC_Power_IE_Sma; (*Current run state of the state machine*)
		bLastEnable : BOOL; (*Save of the last enable*)
	END_STRUCT;
	JS_MC_Power_IE_Sma : 
		( (*JS_MC_Power state machine*)
		enPOWER_W4_DISABLED,
		enPOWER_W4_INIT,
		enPOWER_W4_WAIT_INIT_DONE,
		enPOWER_W4_POWERON,
		enPOWER_W4_ENABLED,
		enPOWER_W4_SHUTDOWN,
		enPOWER_W4_ERROR
		);
END_TYPE

(*For the Reference*)

TYPE
	JS_MC_Reference_IS : 	STRUCT  (*internal structure for JS_MC_Reference*)
		enRunState : JS_MC_Reference_IE_Sma; (*Current run state of the state machine*)
		bOldExecute : BOOL; (*Save of the last execute state*)
		instEplSDOWrite : EplSDOWrite; (*SDO write instance*)
		instEplSDORead : EplSDORead; (*SDO read instance*)
		uiErrorID_Intern : UINT; (*Intern ErrorID*)
		tmpPara : UDINT; (*temp variable*)
		activeRefMode : USINT; (*temp variable for reference method*)
	END_STRUCT;
	JS_MC_Reference_IE_Sma : 
		( (*JS_MC_Reference state machine*)
		enREF_WAIT_FOR_NEXT_EXECUTION,
		enREF_INIT_REFERENCE,
		enREF_SET_ROT_REF_SPEED_INIT,
		enREF_SET_ROT_REF_SPEED_VERIFY,
		enREF_SET_ROT_Z_SPEED_INIT,
		enREF_SET_ROT_Z_SPEED_VERIFY,
		enREF_SET_REF_METHOD_INIT,
		enREF_SET_REF_METHOD_VERIFY,
		enREF_WAIT_FOR_REFERENCE,
		enREF_STOPMOTION_INIT,
		enREF_STOPMOTION_VERIFY,
		enREF_UPDATE_POSITION,
		enREF_COMPLETE_FINISHED,
		enREF_WAIT_MODE_OF_OPERATION_SET,
		enREF_ABORTED,
		enREF_ERROR
		);
END_TYPE

(*For the Stop*)

TYPE
	JS_MC_Stop_IS : 	STRUCT  (*internal structure for JS_MC_Stop*)
		enRunState : JS_MC_Stop_IE_Sma; (*Current run state of the statemachine*)
		bOldExecute : BOOL; (*Save of the last execute state*)
		udiWaitResponseTimer : UDINT; (*Timer for response time*)
		uiErrorID_Intern : UINT; (*Internal ErrorID*)
	END_STRUCT;
	JS_MC_Stop_IE_Sma : 
		( (*JS_MC_Stop state machine*)
		enSTOP_WAIT_FOR_NEXT_EXECUTION,
		enSTOP_INIT_STOP,
		enSTOP_START_STOP,
		enSTOP_WAIT_RESPONSE_TIME,
		enSTOP_WAIT_FOR_FINISH_STOP,
		enSTOP_ABORT,
		enSTOP_ERROR
		);
END_TYPE

(*For the Halt*)

TYPE
	JS_MC_Halt_IS : 	STRUCT  (*internal structure for JS_MC_Halt*)
		enRunState : JS_MC_Halt_IE_Sma; (*Current run state of the state machine*)
		bOldExecute : BOOL; (*Save of the last execute state*)
		udiWaitResponseTimer : UDINT; (*Timer for response time*)
		uiErrorID_Intern : UINT; (*Intern ErrorID*)
	END_STRUCT;
	JS_MC_Halt_IE_Sma : 
		( (*JS_MC_Halt state machine*)
		enHALT_WAIT_FOR_NEXT_EXECUTION,
		enHALT_INIT_HALT,
		enHALT_START_HALT,
		enHALT_WAIT_RESPONSE_TIME,
		enHALT_WAIT_FOR_FINISH_HALT,
		enHALT_ABORT,
		enHALT_ERROR
		);
END_TYPE

(*For the Reset*)

TYPE
	JS_MC_Reset_IS : 	STRUCT  (*internal structure for JS_MC_Reset*)
		enRunState : JS_MC_Reset_IE_Sma; (*Current runstate of the state machine*)
		bOldExecute : BOOL; (*Save of the last execute state*)
	END_STRUCT;
	JS_MC_Reset_IE_Sma : 
		( (*JS_MC_Reset state machine*)
		enRESET_WAIT_FOR_NEXT_EXECUTION,
		enRESET_INIT_RESET,
		enRESET_START_RESET,
		enRESET_WAIT_FOR_FINISH_RESET
		);
END_TYPE

(*For the JogVelocity*)

TYPE
	JS_MC_JogVelocity_IS : 	STRUCT  (*internal structure for JS_MC_JogVelocity*)
		enRunState : JS_MC_JogVelocity_IE_Sma; (*Current run state of the state machine*)
		bLastEnable : BOOL; (*Save of the laste enable signal*)
		udiWaitResponseTimer : UDINT; (*Timer for response time*)
		instEplSDOWrite : EplSDOWrite; (*SDO write instance*)
		instEplSDORead : EplSDORead; (*SDO read instance*)
		uiErrorID_Intern : UINT; (*Intern ErrorID*)
	END_STRUCT;
	JS_MC_JogVelocity_IE_Sma : 
		( (*JS_MC_JogVelocity state machine*)
		enJOGVEL_DISABLED,
		enJOGVEL_INIT,
		enJOGVEL_START,
		enJOGVEL_WAIT_RESPONSE_TIME,
		enJOGVEL_RUN,
		enJOGVEL_FINISH,
		enJOGVEL_ABORTED,
		enJOGVEL_ERROR
		);
END_TYPE

(*For the MoveAbsolute*)

TYPE
	JS_MC_MoveAbsolute_IS : 	STRUCT  (*internal structure for JS_MC_MoveAbsolute*)
		enRunState : JS_MC_MoveAbsolute_IE_Sma; (*Current run state of the state machine*)
		bOldExecute : BOOL; (*Save of the last execute state*)
		uiErrorID_Intern : UINT; (*Internal ErrorID*)
	END_STRUCT;
	JS_MC_MoveAbsolute_IE_Sma : 
		( (*JS_MC_MoveAbsolute state machine*)
		enMOVABS_IDLE,
		enMOVABS_INIT,
		enMOVABS_START,
		enMOVABS_WAIT_ACK,
		enMOVABS_MOVE,
		enMOVABS_ABORT,
		enMOVABS_ERROR
		);
END_TYPE

(*For the MoveRelative*)

TYPE
	JS_MC_MoveRelative_IS : 	STRUCT  (*internal structure for JS_MC_MoveRelative*)
		enRunState : JS_MC_MoveRelative_IE_Sma; (*Current run state of the state machine*)
		bOldExecute : BOOL; (*Save of the last execute state*)
		uiErrorID_Intern : UINT; (*Internal ErrorID*)
	END_STRUCT;
	JS_MC_MoveRelative_IE_Sma : 
		( (*JS_MC_MoveRelative state machine*)
		enMOVREL_IDLE,
		enMOVREL_INIT,
		enMOVREL_START,
		enMOVREL_WAIT_ACK,
		enMOVREL_MOVE,
		enMOVREL_ABORT,
		enMOVREL_ERROR
		);
END_TYPE

(*For the MoveCyclicPosition*)

TYPE
	JS_MC_MoveCyclicPosition_IS : 	STRUCT  (*internal structure for JS_MC_MoveCyclicPosition*)
		enRunState : JS_MC_MoveCyclicPosition_IE_Sma; (*Current run state of the internal state machine*)
		bLastEnable : BOOL; (*Save of the laste enable signal*)
	END_STRUCT;
	JS_MC_MoveCyclicPosition_IE_Sma : 
		( (*JS_MC_MoveCyclicPosition state machine*)
		enCYCSYNC_DISABLED,
		enCYCSYNC_RUN,
		enCYCSYNC_ABORT,
		enCYCSYNC_ERROR
		);
END_TYPE

(*For the ForceCalibration*)

TYPE
	JS_MC_ForceCalibration_IS : 	STRUCT  (*internal structure for JS_MC_ForceCalibration*)
		enRunState : JS_MC_ForceCalibration_IE_Sma; (*Current run state of the state machine*)
		bOldExecute : BOOL; (*Save of the last execute state*)
		instEplSDOWrite : EplSDOWrite; (*SDO write instance*)
		instEplSDORead : EplSDORead; (*SDO read instance*)
		tmpPara : UDINT; (*temp variable*)
		uiErrorID_Intern : UINT; (*Internal ErrorID*)
		clearOldCalib : BOOL;
	END_STRUCT;
	JS_MC_ForceCalibration_IE_Sma : 
		( (*JS_MC_ForceCalibration  state machine*)
		enFC_WAIT_FOR_NEXT_EXECUTION,
		enFC_INIT_FC,
		enFC_VERIFY_FC_MODE,
		enFC_VERIFY_FC_MODE_INIT,
		enFC_SET_FC_MODE_VERIFY,
		enFC_SET_FC_MODE_INIT,
		enFC_SET_MODE,
		enFC_PARA_INIT,
		enFC_PARA_VERIFY,
		enFC_REFERENCE_INIT,
		enFC_REFERENCE_VERIFY,
		enFC_REFERENCE_END,
		enFC_MOVE_STARTPOS_INIT,
		enFC_MOVE_STARTPOS_VERIFY,
		enFC_MOVE_STARTPOS_END,
		enFC_SET_WAY_VERIFY_RESET,
		enFC_SET_WAY_VERIFY,
		enFC_SET_WAY_END,
		enFC_FORCE_CALIBRATION_VERIFY,
		enFC_WAIT_FOR_FC,
		enFC_FORCE_CALIBRATION_VALID,
		enFC_STOPMOTION_INIT,
		enFC_STOPMOTION_VERIFY,
		enFC_RESTORE_POSITION_INIT,
		enFC_RESTORE_POSITION_VERIFY,
		enFC_UPDATE_POSITION_INIT,
		enFC_UPDATE_POSITION_WAIT,
		enFC_COMPLETE_FINISHED,
		enFC_ABORTED,
		enFC_ERROR
		);
END_TYPE

(*For the ReadStatus*)

TYPE
	JS_MC_ReadStatus_IS : 	STRUCT  (*internal structure for JS_MC_ReadStatus*)
		bOldEnable : BOOL; (*Save of the last enable signal*)
	END_STRUCT;
END_TYPE

(*For the ReadActualCurrent*)

TYPE
	JS_MC_ReadActualCurrent_IS : 	STRUCT  (*internal structure for JS_MC_ReadActualCurrent*)
		enRunState : JS_MC_ReadActualCurrent_IE_Sma; (*Current run state of the internal state machine*)
		bLastEnable : BOOL; (*Save of the laste enable signal*)
	END_STRUCT;
	JS_MC_ReadActualCurrent_IE_Sma : 
		( (*JS_MC_ReadActualCurrent state machine*)
		enRDCURR_DISABLED,
		enRDCURR_RUN,
		enRDCURR_ERROR
		);
END_TYPE

(*For the ReadAxisError*)

TYPE
	JS_MC_ReadAxisError_IS : 	STRUCT  (*internal structure for JS_MC_ReadAxisError*)
		enRunStateAxisNumber : JS_MC_ReadAxisError_IE_AxisNrSma; (*Current run state of the internal state machine for Axis number readout*)
		enRunStateErrorText : JS_MC_ReadAxisError_IE_ErrTxtSma; (*Current run state of the internal state machine for Text readout*)
		udiIdent : UDINT; (*Ident for Error Text object*)
		udiIdent_FBText : UDINT; (*Ident for Function Block Text object*)
		instEplSDORead : EplSDORead; (*SDO read instance*)
		instDatObjInfo : DatObjInfo;
		instDatObjRead : DatObjRead;
		udiAxisNumber : UDINT; (*Error or warning number of the xenax controller*)
		bOldEnable : BOOL; (*Save of the last enable state*)
		sOldDataObjectName : STRING[12]; (*Save of the last error text object name*)
		bOldAcknowledge : BOOL; (*Save of the last acknowledge state*)
		bIdent_FBText_Readout : BOOL; (*Ident-Number for function block text already read out*)
	END_STRUCT;
	JS_MC_ReadAxisError_IE_ErrTxtSma : 
		( (*JS_MC_ReadAxisError state machine (for error number readout)*)
		enRDAXISERR_W4_GET_ID_INIT,
		enRDAXISERR_W4_GET_ID_VERIFY,
		enRDAXISERR_W4_GET_ERRTXT_INIT,
		enRDAXISERR_W4_GET_ERRTXT_VERIFY,
		enRDAXISERR_W4_WAIT,
		enRDAXISERR_W4_ERRTXT_ERROR
		);
	JS_MC_ReadAxisError_IE_AxisNrSma : 
		( (*JS_MC_ReadAxisError state machine (for error text readout)*)
		enRDAXISERR_W4_READ_SDO_INIT,
		enRDAXISERR_W4_READ_SDO_VERIFY,
		enRDAXISERR_W4_WAIT2,
		enRDAXISERR_W4_AXISNR_ERROR
		);
	JS_MC_ErrorRecord : 	STRUCT  (*Error Record (output of JS_MC_ReadAxisError)*)
		Number : UINT; (*Error or warning number*)
		ErrorSource : USINT; (*Error Source (Axis or a specific function block):<br>
jsmcAXIS<br>
jsmcFB_JS_MC_CYCLICIN<br>
jsmcFB_JS_MC_POWER<br>
jsmcFB_JS_MC_REFERENCE<br>
jsmcFB_JS_MC_MOVEABSOLUTE<br>
jsmcFB_JS_MC_MOVERELATIVE<br>
jsmcFB_JS_MC_MOVECYCLICPOSITION<br>
jsmcFB_JS_MC_STOP<br>
jsmcFB_JS_MC_HALT<br>
jsmcFB_JS_MC_AXISERRORCOLLECTOR<br>
jsmcFB_JS_MC_READAXISERROR<br>
jsmcFB_JS_MC_READPARAMETER<br>
jsmcFB_JS_MC_WRITEPARAMETER<br>
jsmcFB_JS_MC_JOGVELOCITY<br>
jsmcFB_JS_MC_READACTUALCURRENT<br>
jsmcFB_JS_MC_READDIGITALINPUT<br>
jsmcFB_JS_MC_READDIGITALOUTPUT<br>
jsmcFB_JS_MC_WRITEDIGITALOUTPUT*)
		ErrorType : USINT; (*Error Type (Axis or function block error/warning):<br>
jsmcAXIS_ERROR<br>
jsmcAXIS_WARNING<br>
jsmcFB_ERROR*)
	END_STRUCT;
END_TYPE

(*For the AxisErrorCollector*)

TYPE
	JS_MC_AxisErrorCollector_IS : 	STRUCT  (*internal structure for JS_MC_AxisErrorCollector*)
		enRunState : JS_MC_AxisErrorCollector_IE_Sma; (*Current run state of the internal state machine*)
		bLastEnable : BOOL; (*Save of the last enable state*)
	END_STRUCT;
	JS_MC_AxisErrorCollector_IE_Sma : 
		( (*JS_MC_AxisErrorCollector state machine*)
		enAXISERRCOL_W4_ENABLE,
		enAXISERRCOL_W4_ERROR
		);
END_TYPE

(*For the JS_MC_ReadParameter*)

TYPE
	JS_MC_ReadParameter_IS : 	STRUCT  (*internal structure for JS_MC_ReadParameter*)
		enRunState : JS_MC_ReadParameter_IE_Sma; (*Current run state of the internal state machine*)
		instEplSDORead : EplSDORead; (*SDO read object*)
		uiLastDataObject : UINT; (*Save of the last data object to be read out*)
		usiLastSubID : USINT; (*Save of the last SubID to be read out*)
		usiLastDataLength : USINT; (*Save of the last data length to be read out*)
		bLastEnable : BOOL; (*Save of the last enable state*)
	END_STRUCT;
	JS_MC_ReadParameter_IE_Sma : 
		( (*JS_MC_ReadParameter state machine*)
		enRDPAR_DISABLED,
		enRDPAR_READOUT_INIT,
		enRDPAR_READOUT,
		enRDPAR_ERROR
		);
END_TYPE

(*For the JS_MC_ReadDigitalInput*)

TYPE
	JS_MC_ReadDigitalInput_IS : 	STRUCT  (*internal structure for JS_MC_ReadDigitalInput*)
		enRunState : JS_MC_ReadDigitalInput_IE_Sma; (*Current run state of the internal state machine*)
		bLastEnable : BOOL; (*Save of the laste enable signal*)
	END_STRUCT;
	JS_MC_ReadDigitalInput_IE_Sma : 
		( (*JS_MC_ReadDigitalInput state machine*)
		enRDDIGI_DISABLED,
		enRDDIGI_RUN,
		enRDDIGI_ERROR
		);
END_TYPE

(*For the JS_MC_ReadDigitalOutput*)

TYPE
	JS_MC_ReadDigitalOutput_IS : 	STRUCT  (*internal structure for JS_MC_ReadDigitalOutput*)
		enRunState : JS_MC_ReadDigitalOutput_IE_Sma; (*Current run state of the internal state machine*)
		bLastEnable : BOOL; (*Save of the laste enable signal*)
	END_STRUCT;
	JS_MC_ReadDigitalOutput_IE_Sma : 
		( (*JS_MC_ReadDigitalInput state machine*)
		enRDDIGO_DISABLED,
		enRDDIGO_RUN,
		enRDDIGO_ERROR
		);
END_TYPE

(*For the JS_MC_SetPDO*)

TYPE
	JS_MC_SetPDO_IS : 	STRUCT  (*internal structure for JS_MC_SetPDO*)
		enRunState : JS_MC_SetPDO_IE_Sma; (*Current run state of the internal state machine*)
		bOldExecute : USINT; (*Save of the last execute state*)
	END_STRUCT;
	JS_MC_SetPDO_IE_Sma : 
		( (*JS_MC_SetPDOr state machine*)
		enSetPDO_WAIT_FOR_NEXT_EXECUTION,
		enSetPDO_CHECK,
		enSetPDO_EXECUTION,
		enSetPDO_ERROR
		);
END_TYPE

(*For the JS_MC_WriteParameter*)

TYPE
	JS_MC_WriteParameter_IS : 	STRUCT  (*internal structure for JS_MC_WriteParameter*)
		enRunState : JS_MC_WriteParameter_IE_Sma; (*Current run state of the internal state machine*)
		instEplSDOWrite : EplSDOWrite; (*SDO write object*)
		bOldExecute : USINT; (*Save of the last execute state*)
		uiErrorID_Intern : UINT; (*Internal error number*)
	END_STRUCT;
	JS_MC_WriteParameter_IE_Sma : 
		( (*JS_MC_WriteParameter state machine*)
		enWRPAR_WAIT_FOR_NEXT_EXECUTION,
		enWRPAR_WRITE_INIT,
		enWRPAR_WRITE_VERIFY,
		enWRPAR_ERROR
		);
END_TYPE

(*For the WriteDigitalOutput*)

TYPE
	JS_MC_WriteDigOut_IS : 	STRUCT  (*internal structure for JS_MC_WriteDigOut*)
		enRunState : JS_MC_WriteDigOut_IE_Sma; (*Current run state of the internal state machine*)
		udiDigitalOutput : UDINT; (*Save of the output register (for shifting)*)
		bOldExecute : BOOL; (*Save of the last execute state*)
		uiErrorID_Intern : UINT; (*Intern ErrorID*)
	END_STRUCT;
	JS_MC_WriteDigOut_IE_Sma : 
		( (*JS_MC_WriteDigOut state machine*)
		enWROUT_WAIT_FOR_NEXT_EXECUTION,
		enWROUT_FINISH_WRITE,
		enWROUT_INIT_WRITE,
		enWROUT_ERROR
		);
END_TYPE

(*For the HandleModeOfOperation*)

TYPE
	JS_MC_HndlModeOfOperation_IS : 	STRUCT 
		enRunState : JS_MC_HndlModeOfOperation_IE_Sma; (*Current run state of the internal state machine*)
		instEplSDOWrite : EplSDOWrite; (*SDO write instance*)
		instEplSDORead : EplSDORead; (*SDO read instance*)
		siModeSet : SINT; (*Currently set mode of operation*)
		siModeToSet : SINT; (*Desired mode of operation to set*)
	END_STRUCT;
	JS_MC_HndlModeOfOperation_IE_Sma : 
		( (*JS_MC_HndlModeOfOperation_IE_Sma state machine*)
		enHMOO_IDLE,
		enHMOO_SET_MODE_INIT,
		enHMOO_SET_MODE_WAIT,
		enHMOO_VERIFY_MODE_INIT,
		enHMOO_VERIFY_MODE_WAIT
		);
END_TYPE
